@@ -1,13 +1,18 @@
-from actionpack import Action
-
 from concurrent.futures import ThreadPoolExecutor
 from concurrent.futures import as_completed
 from functools import reduce
-from multiprocessing.pool import ThreadPool
+from typing import Generic
+from typing import Iterator
+from typing import Tuple
+
+from actionpack.action import Name
+from actionpack.action import Outcome
+from actionpack.action import Result
+from actionpack import Action
 
 
-class Procedure:
-    def __init__(self, *actions: Action):
+class Procedure(Generic[Name, Outcome]):
+    def __init__(self, *actions: Action[Name, Outcome]):
         self.actions = actions
         self._actions = iter(self.actions)
 
@@ -20,7 +25,12 @@ class Procedure:
         except Exception as e:
             raise e
 
-    def execute(self, max_workers: int=5, should_raise: bool=False, synchronously: bool=True):
+    def execute(
+        self,
+        max_workers: int = 5,
+        should_raise: bool = False,
+        synchronously: bool = True
+    ) -> Iterator[Result[Outcome]]:
         if synchronously:
             for action in self.actions:
                 yield action.perform(should_raise=should_raise) if should_raise else action.perform()
@@ -31,11 +41,10 @@ class Procedure:
                     yield future.result()
 
     def __repr__(self):
-        for action in self.actions:
-            header = f'\nProcedure for performing the following Actions:\n'
-            bullet = '  * '
-            actions = reduce(lambda a, b: str(a) + f'\n{bullet}' + str(b), self.actions)
-            return header + bullet + actions
+        header = '\nProcedure for performing the following Actions:\n'
+        bullet = '  * '
+        actions = reduce(lambda a, b: str(a) + f'\n{bullet}' + str(b), self.actions)
+        return header + bullet + str(actions)
 
     def __iter__(self):
         return self._actions
@@ -46,10 +55,11 @@ class Procedure:
         except StopIteration:
             self._actions = iter(self.actions)
 
-    class NotAnAction(Exception): pass
+    class NotAnAction(Exception):
+        pass
 
 
-class KeyedProcedure(Procedure):
+class KeyedProcedure(Procedure[Name, Outcome]):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -61,7 +71,12 @@ class KeyedProcedure(Procedure):
                 msg = f'All {self.__class__.__name__} Actions must have a name: {str(action)}'
                 raise KeyedProcedure.UnnamedAction(msg)
 
-    def execute(self, max_workers: int=5, should_raise: bool=False, synchronously: bool=True):
+    def execute(
+        self,
+        max_workers: int = 5,
+        should_raise: bool = False,
+        synchronously: bool = True
+    ) -> Iterator[Tuple[Name, Result[Outcome]]]:
         if synchronously:
             for action in self.actions:
                 yield (action.name, action.perform(should_raise=should_raise)) \
@@ -72,4 +87,5 @@ class KeyedProcedure(Procedure):
                 for future in as_completed(futures):
                     yield (futures[future].name, future.result())
 
-    class UnnamedAction(Exception): pass
+    class UnnamedAction(Exception):
+        pass
