@@ -1,5 +1,6 @@
 from io import BytesIO
 from io import StringIO
+from typing import Callable
 
 from actionpack import Action
 from actionpack.action import Name
@@ -10,23 +11,24 @@ class FakeAction(Action[Name, Outcome]):
 
     result = 'Performing Action.'
 
-    def __init__(self, name: Name = None, exception=None):
+    def __init__(
+        self,
+        name: Name = None,
+        instruction_provider: Callable = None,
+    ):
         self.name = name
-        self.exception = exception
+        self.instruction_provider = instruction_provider
         self.state = {'this': 'state'}
 
     def instruction(self) -> str:
-        return self.result
-
-    def validate(self):
-        if self.exception:
-            raise self.exception
-        return super().validate()
+        return self.instruction_provider() if self.instruction_provider else self.result
 
 
 class FakeFile:
-    def __init__(self, contents: bytes = bytes(), mode: str = None):
-        self.buffer = BytesIO(contents)
+    def __init__(self, contents=None, mode: str = None):
+        if not contents:
+            contents = bytes()
+        self.buffer = BytesIO(contents) if isinstance(contents, bytes) else StringIO(contents)
         self.buffer.read()
         self.mode = mode
 
@@ -34,8 +36,8 @@ class FakeFile:
         self.buffer.seek(0)
         return self.buffer.read()
 
-    def write(self, data: bytes):
-        if self.mode == 'wb':
+    def write(self, data):
+        if self.mode in ['wb', 'w']:
             self.buffer.seek(0)
             self.buffer.truncate()
         self.buffer.write(data)
