@@ -1,19 +1,23 @@
-from actionpack import Action
+from collections.abc import Iterable
+from unittest import TestCase
+
 from actionpack import KeyedProcedure
 from actionpack import Procedure
+from actionpack.action import Result
 from tests.actionpack import FakeAction
 from tests.actionpack import FakeFile
-from tests.actionpack.actions import FakeWriteBytes
+from tests.actionpack.actions import FakeWrite
 
-from collections.abc import Iterable
-from oslash import Left
-from oslash import Right
-from unittest import TestCase
+
+def raise_failure():
+    raise Exception('something went wrong :/')
 
 
 success = FakeAction(name='success')
-failure = FakeAction(name='failure',
-                     exception=Exception('something went wrong :/'))
+failure = FakeAction(
+    name='failure',
+    instruction_provider=raise_failure
+)
 
 
 def assertIsIterable(possible_collection):
@@ -35,14 +39,14 @@ class ProcedureTest(TestCase):
         results = self.procedure.execute()
 
         assertIsIterable(results)
-        self.assertIsInstance(next(results), Right)
-        self.assertIsInstance(next(results), Left)
+        self.assertIsInstance(next(results), Result)
+        self.assertIsInstance(next(results), Result)
 
     def test_Procedure_execution_can_raise(self):
         results = self.procedure.execute(should_raise=True)
 
         assertIsIterable(results)
-        self.assertIsInstance(next(results), Right)
+        self.assertIsInstance(next(results), Result)
         with self.assertRaises(Exception):
             next(results)
 
@@ -51,20 +55,20 @@ class ProcedureTest(TestCase):
             Procedure('wut.', failure).validate()
 
     def test_can_execute_Procedure_asynchronously(self):
-        file = FakeFile(mode='w')
+        file = FakeFile()
 
         question = b' How are you?'
         wellwish = b' I hope you\'re well.'
 
-        action1 = FakeWriteBytes(file, question, delay=0.2)
-        action2 = FakeWriteBytes(file, wellwish, delay=0.1)
+        action1 = FakeWrite[str, int](file, question, delay=0.2)
+        action2 = FakeWrite[str, int](file, wellwish, delay=0.1)
 
-        procedure = Procedure(action1, action2)
+        procedure = Procedure[str, int](action1, action2)
         results = procedure.execute(should_raise=True, synchronously=False)
 
         assertIsIterable(results)
-        self.assertIsInstance(next(results), Right)
-        self.assertIsInstance(next(results), Right)
+        self.assertIsInstance(next(results), Result)
+        self.assertIsInstance(next(results), Result)
 
         # NOTE: the wellwish precedes since the question took longer
         self.assertEqual(file.read(), wellwish + question)
@@ -73,10 +77,10 @@ class ProcedureTest(TestCase):
 class KeyedProcedureTest(TestCase):
 
     def test_can_create_KeyedProcedure(self):
-        results = KeyedProcedure(success, failure).execute()
+        results = KeyedProcedure[str, str](success, failure).execute()
         results_dict = dict(results)
 
-        self.assertIsInstance(results_dict[success.name], Right)
+        self.assertIsInstance(results_dict[success.name], Result)
 
     def test_KeyedProcedure_execution_can_raise(self):
         results = KeyedProcedure(success, failure).execute(should_raise=True)
@@ -87,8 +91,12 @@ class KeyedProcedureTest(TestCase):
             next(results)
 
     def test_can_create_KeyedProcedure_from_Actions_named_using_any_scriptable_type(self):
-        action1, action2, action3 = FakeAction(), FakeAction(exception=Exception()), FakeAction()
+        action1 = FakeAction[int, str]()
+        action2 = FakeAction[bool, str](instruction_provider=raise_failure)
+        action3 = FakeAction[float, str]()
+
         key1, key2, key3 = 1, False, 1.01
+
         results = KeyedProcedure(
             action1.set(name=key1),
             action2.set(name=key2),
@@ -96,11 +104,10 @@ class KeyedProcedureTest(TestCase):
         ).execute()
         results_dict = dict(results)
 
-        self.assertIsInstance(results_dict[key1], Right)
-        self.assertIsInstance(results_dict[key2], Left)
-        self.assertIsInstance(results_dict[key3], Right)
+        self.assertIsInstance(results_dict[key1].value, str)
+        self.assertIsInstance(results_dict[key2].value, Exception)
+        self.assertIsInstance(results_dict[key3].value, str)
 
     def test_can_validate_KeyedProcedure(self):
         with self.assertRaises(KeyedProcedure.UnnamedAction):
             KeyedProcedure(FakeAction(), failure)
-
